clear; clc;
load('patchMaster.mat');
adj_travelTimes(:,1) = [patchMaster.Global.adj_travelTimes.smooth_mean_adj]';
adj_travelTimes(:,2) = [patchMaster.Global.adj_travelTimes.smooth_STD_adj]';

% Parameters
m = patchMaster.Global.trials_perSession; % number of trials (empirical)
s = 1000; % number of simulations

initialReward = 0.2300;     % the value from the Duke data
rewardDecrement = 0.0141; % (empirical)

bootstrap_size = 10001;
Smax = ceil(initialReward / rewardDecrement); % maximum number of stay decisions in one patch
nS = Smax; % # of diff patch residence lengths to use in each simulation
g(1:m,1:nS) = NaN; % preallocating rewards per simulation
E(1:m,1:nS,1:s) = NaN; % preallocating total reward
Scount(1:nS) = 0;
youWannaSave = 1;
youWannaPlot = 1;
noise_analysis = 1;

% subjects to simulate

numCycles = ceil(m/Smax); cycleCount = 1;
for c = 1:numCycles
    Sselect(cycleCount:cycleCount+Smax-1) = randperm(Smax,Smax);
    cycleCount = cycleCount + Smax;
end
   
for i = 1:s % for each simulation...

    for j = 1:m % for each patch...
        
        Sm = Sselect(j); % the patch residence length on this trial
        t = normrnd(adj_travelTimes(Sm,1),adj_travelTimes(Sm,2)); % empirical adj travel time
        if t < 0, t = 0; end        
        Scount(Sm) = Scount(Sm)+1;
        
        for k = 1:Sm
            if  k == 1
                gAccumulated = initialReward;
            elseif k > 1
                gAccumulated = gAccumulated + (initialReward - (k * rewardDecrement));
                % the total reward as a function of # of rewards obtained
                if gAccumulated < 0
                    gAccumulated = 0;
                end
            end
        end
        
        g(Scount(Sm),Sm) = gAccumulated/((t*Sm) + Sm); % the reward rate on this trial
        
    end
    Scount(1:nS) = 0; % reset this variable for the next simulation
    E(1:size(g,1),1:size(g,2),i) = g; % the reward rates in this simulation
    if i/10 == round(i/10)
        clc; disp(['(1/2): ' num2str(round((i/s)*100)) '%']);
    end
    
end
   
E_trialMean = nanmean(E,1);
for sim = 1:size(E_trialMean,3)
    sim_gAccumulated(sim,1:Smax) = E_trialMean(1,1:Smax,sim);
end
for m = 1:length(sim_gAccumulated)
    sim_max_g(m,1) = find(sim_gAccumulated(m,:)==max(sim_gAccumulated(m,:)));
end
E_simulationMean = nanmean(E,3);
E_total = nanmean(E_simulationMean,1);
X(1:nS) = (1:nS);
Xmax = X(find(E_total == max(E_total)));

numBins = 10;

optimalThreshold_sim = Xmax; 

if youWannaPlot

plot(X,E_total,'LineWidth',2,'color','b'); xlabel('# trials in patch'); 
ylabel('Reward harvest rate'); hold on;

end

if youWannaSave
    
        patchMaster.E.simData.sim_gAccumulated = [];
        patchMaster.E.simData.sim_max_g = [];
        patchMaster.E.simData.mean_optimal = [];
        patchMaster.E.simData.std_optimal = [];
        patchMaster.E.simData.sim_gAccumulated = sim_gAccumulated;
        patchMaster.E.simData.sim_max_g = sim_max_g;
        patchMaster.E.simData.mean_optimal = nanmean(sim_max_g);
        patchMaster.E.simData.std_optimal = nanstd(sim_max_g);
        save('patchMaster','patchMaster')
%         clearvars patchMaster
        
end

if noise_analysis
    
    interpolation = 20;
    sigmaMax = length(E_total)*interpolation;
    sigmaCount = 0;
    X_interp = 0:1/interpolation:max(X); X_interp = X_interp(1:end-1);
    interp_minCount = 1;
    interp_maxCount = interpolation;
    
    for i = 0:length(E_total)-1
        if i == 0
            E_interp_segment = 0:E_total(i+1)/interpolation:E_total(i+1);
        else
            E_interp_segment = E_total(i):(E_total(i+1)-E_total(i))/interpolation:E_total(i+1);
        end
        E_interp_segment = E_interp_segment(1:interpolation);
        E_interp(interp_minCount:interp_maxCount) = E_interp_segment;
        interp_minCount = interp_minCount + interpolation; interp_maxCount = ...
            interp_maxCount + interpolation;
    end
    
    
    for sigma = 1:sigmaMax
        sigmaCount = sigmaCount + 1;
        noisyValue = NaN;
        for center = 1:length(X_interp)
            noisyReward = generate_Gaussian(E_interp,center,sigma,0);
            noisyValue(center,1) = trapz(noisyReward);        
        end
        try
            optimalThreshold(1,sigmaCount) = X_interp(find(noisyValue==max(noisyValue)));
        catch
            optimalThreshold(1,sigmaCount) = NaN;
        end
        clc; disp(['(1/2): ' '100%']); '\n';
        disp(['(2/2): ' num2str(round((sigma/sigmaMax)*100)) '%']);
    end
    
    if youWannaPlot
        figure % gaussian noise plot
        plot(X_interp,optimalThreshold,'LineWidth',2,'color','b'); ...
            xlabel('Uncertainty'); ylabel('Optimal Threshold'); hold on;
    end
    
end

% To calculate the uncertainty-adjusted optimal threshold:
uncertainty = patchMaster.E.patchLength_STD;
index = find(abs(X_interp - uncertainty)==min(abs(X_interp - uncertainty)));
uncertaintyAdj_optimalThreshold = optimalThreshold(index);

if youWannaSave && noise_analysis
    
    disp('saving ... ');
    load('patchMaster.mat')
    
    patchMaster.E.simData.E_interp = E_interp;
    patchMaster.E.simData.E_simulationMean = E_simulationMean;
    patchMaster.E.simData.E_total = E_total;
    patchMaster.E.simData.nS = nS;
    patchMaster.E.simData.optimalThreshold = optimalThreshold;
    patchMaster.E.simData.X = X;
    patchMaster.E.simData.X_interp = X_interp;
    patchMaster.E.simData.Xmax = Xmax;
    patchMaster.E.optimal_uncertaintyAdj_threshold = uncertaintyAdj_optimalThreshold;
    
    save('patchMaster','patchMaster')
    disp('data saved');
    
end

clear all; clc;