function xPatch
clear; clc;
global dataFile dataFolder dataSubFolder sessionCount charnovData mean_patchTime SE_patchTime ...
    mean_deliberationTime mean_rewardDecrement SE_rewardDecrement youWannaSave mean_rewardRate ...
    STD_patchTime trialData patches patchMaster dataFilename decrement rootFolder

rootFolder = cd;
dataFolder = [cd '\Data\Patch\ernie'];
sessionCount = 1;
youWannaSave = 1;
charnovData = struct; patchMaster = struct; trialData = struct; patches = struct;
dCharnov

    function dCharnov % open the data
        
        d = dir(dataFolder); % list the contents of the data folder
        dataFolderList = [d(:).isdir];
        dataFolderNames = {d(dataFolderList).name};
        pause(0.5)
        disp('Processing ...'); pause(0.5)
        
        for i = 3:length(find(dataFolderList==1)) % see 'dataFolderNames' for why this starts at 3
            
            dataSubFolder = [dataFolder '\' dataFolderNames{i}];
            % generates a string that references each folder
            
            cd(dataSubFolder) % opens that folder
            dataDir = dir(dataSubFolder);
            for j_dataSession = 3:length(dataDir)
                [filepath,name,ext] = fileparts(dataDir(j_dataSession).name); %#ok<*ASGLU>
                if strcmp(ext,'.mat') % will only open .mat files
                    dataFile = load(dataDir(j_dataSession).name);
                    dataFilename = name;
                    wCharnov
                    % add the data from that file to the global wrapped data file (see function below)
                end
                if i == length(dataFolderNames) && j_dataSession == length(dataDir)
                    aCharnov
                end
            end
            
        end
        
    end

    function wCharnov % wrap data from individual charnov sessions
        
        % our goal is to get reward rate for each patch
        foragingrwd = [dataFile.data(1:end).foragingrwd];
        sessionLength = length(dataFile.data);
        numPatches = length(find(foragingrwd(2:end) > foragingrwd(1:end-1))) + 1;
        patch_idx(1) = 1;
        patch_idx(2:numPatches) = find(foragingrwd(2:end) > foragingrwd(1:end-1)) + 1;
        patch_length(1:numPatches-1) = patch_idx(2:end) - patch_idx(1:end-1);
        patch_length(numPatches) = length(dataFile.data) - patch_idx(numPatches);
        for i = 1:numPatches
            patchTime(i) = patch_length(i);
        end
        for j = 1:(sessionLength-1)
            rewardDecrement(j) = foragingrwd(j) - foragingrwd(j+1);
        end
        try
            handlingTime = [dataFile.data.handlingtime];
            handlingTime = nanmean(handlingTime);
        catch
            handlingTime = NaN;
        end
        rewardDecrement = rewardDecrement(find(rewardDecrement > 0));
        sessionEnd = dataFile.data(sessionLength).trial_start_time + dataFile.data(sessionLength).ev.trial_over;
        sessionStart = dataFile.data(1).trial_start_time;
        sessionTime = sessionLength;
        maxrwd = max(foragingrwd);
        % then get the total reward from the sesh:
        foragingrwd = nansum([dataFile.data(1:end).foragingrwd]);
        rewardRate = foragingrwd/(sessionTime);
        try
            ITI = nanmean([dataFile.data.iti]);
        catch
            ITI = NaN;
        end
        
        for k = 1:sessionLength
            try
                deliberationTime(k) = dataFile.data(k).ev.fixation_acquired;
            catch
                deliberationTime(k) = NaN;
            end
        end
        % to restrict by handling time and max reward
        if round(handlingTime,2) == 0.3 && round(maxrwd,3) == 0.23;
            charnovData(sessionCount).time_inPatch = nanmean(patchTime);
            charnovData(sessionCount).STD_time_inPatch = nanstd(patchTime);
            charnovData(sessionCount).SE_time_inPatch = nanstd(patchTime)/sqrt(sessionLength);
            charnovData(sessionCount).rewardRate = rewardRate;
            charnovData(sessionCount).deliberationTime = nanmean(deliberationTime);
            charnovData(sessionCount).rewardDecrement = nanmean(rewardDecrement);
            charnovData(sessionCount).SE_rewardDecrement = nanstd(rewardDecrement)/sqrt(sessionLength);
            charnovData(sessionCount).handlingTime = handlingTime;
            charnovData(sessionCount).initialReward = maxrwd;
            charnovData(sessionCount).ITI = ITI;
            charnovData(sessionCount).numTrials = sessionLength;
            charnovData(sessionCount).filename = dataFilename;
            sessionCount = sessionCount + 1;
        end
        
        dataFile.data(1).decrement = NaN;
        dataFile.data(1).rwdi = NaN;
        
        if sessionCount - 1 == 1
            trialData = [dataFile.data];
        else
            trialData = [trialData,dataFile.data];
        end
    end

    function aCharnov
        
        rewardRate = [charnovData.rewardRate];
        time_inPatch = [charnovData.time_inPatch]; % (# stay decisions)
        STD_time_inPatch = [charnovData.STD_time_inPatch];
        SE_time_inPatch = [charnovData.SE_time_inPatch];
        deliberationTime = [charnovData.deliberationTime];
        rewardDecrement = [charnovData.rewardDecrement];
        SE_rewardDecrement = [charnovData.SE_rewardDecrement];
        numTrials = [charnovData.numTrials];
        totalTrials = nansum(numTrials);
        
        for k = 1:length(charnovData)
            weight = numTrials(k) / totalTrials;
            rrWeight(k) = rewardRate(k) * weight;
            tWeight(k) = time_inPatch(k) * weight;
            STDtWeight(k) = STD_time_inPatch(k) * weight;
            SEtWeight(k) = SE_time_inPatch(k) * weight;
            dWeight(k) = deliberationTime(k) * weight;
            rdWeight(k) = rewardDecrement(k) * weight;
            SErdWeight(k) = SE_rewardDecrement(k) * weight;
        end
        
        mean_rewardRate = nansum(rrWeight);
        mean_patchTime = round(nansum(tWeight),4);
        STD_patchTime = nansum(STDtWeight);
        SE_patchTime = round(nansum(SEtWeight),2);
        mean_deliberationTime = nansum(dWeight);
        mean_rewardDecrement = nansum(rdWeight);
        SE_rewardDecrement = nansum(SErdWeight);
        
        for i = 1:length(trialData)
            if trialData(i).foragingrwd < 0
                trialData(i).foragingrwd = 0;
            end
            
        end
        trialRewards = [trialData.foragingrwd];
        decrements = trialRewards(2:end) - trialRewards(1:end-1);
        patchCount = 0; patches = []; leaveIdx = []; timePerTrial = [];
        trialCount = 0;
        for t = 1:length(trialData)
            if trialData(t).LLside == 1
                patchCount = patchCount + 1;
                leaveIdx(patchCount) = t;
                if patchCount == 1
                    patch_foragingrwd = [trialData(1:leaveIdx(patchCount)).foragingrwd];
                    patch_foragingrwd(find(patch_foragingrwd==0)) = 0;
                    patchReward = nansum(patch_foragingrwd);
                else
                    patch_foragingrwd = [trialData(leaveIdx(patchCount - 1):...
                        leaveIdx(patchCount)).foragingrwd];
                    patch_foragingrwd(find(patch_foragingrwd==0)) = 0;
                    patchReward = nansum(patch_foragingrwd);
                end
                patches(patchCount).reward = patchReward;
                fixationError = 0;
                if patchCount == 1
                    patches(patchCount).length = leaveIdx(patchCount) - fixationError - 1;
                    patches(patchCount).patchTime = trialData(t).trial_start_time - ...
                        trialData(1).trial_start_time;                        
                else
                    patches(patchCount).length = leaveIdx(patchCount) - leaveIdx(patchCount - 1)...
                        - fixationError - 1;
                    patches(patchCount).patchTime = trialData(t).trial_start_time - ...
                        trialData(leaveIdx(patchCount-1)).trial_start_time;   
                end
                patches(patchCount).travelTime = trialData(t).ev.iti_begin - ...
                    trialData(t).ev.fixation_acquired;
                patches(patchCount).adj_travelTime = patches(patchCount).travelTime / ...
                    patches(patchCount).patchTime;
            else
                trialCount = trialCount + 1;
                if trialCount ~= 1
                    decrement(trialCount-1,1) = trialData(t).foragingrwd - trialData(t-1).foragingrwd;
                    decrement(trialCount-1,2) = t; 
                    if decrement(trialCount-1,1)>0
                        decrement(trialCount-1,3) = 1;
                    else
                        decrement(trialCount-1,3) = 0;
                    end
                end
            end
        end
        
        for l = 1:length(patches)
            patches(l).timePerTrial = patches(l).patchTime / patches(l).length;
        end
        timePerTrial = [patches.timePerTrial]';
        timePerTrial(find(timePerTrial < 0)) = NaN;
        timePerTrial(find(isinf(timePerTrial))) = NaN;
        
        goodPatches = find([patches.length]>=1)';
        badPatches = find([patches.length]<=0)';
        patches = patches(goodPatches);
        
        patchLengths = [patches.length];
        numPatches = length(patches);
        
        cd(rootFolder)
        try
            load('patchMaster.mat');
        end
%         try
%             load('MVT_charnov_16x10000x1000','optimalThreshold_sim')
%         end
        patchMaster.E.total_numTrials = nansum([patches.length]);
        patchMaster.E.patchLength_mean = nanmean(patchLengths);
        patchMaster.E.patchLength_STD = nanstd(patchLengths);
        patchMaster.E.patchLength_SE = nanstd(patchLengths)/sqrt(numPatches);
        patchMaster.E.actual_totalReward = nansum([trialData.foragingrwd]);
        patchMaster.E.optimal_totalReward = ...
            (0.230 + (0.230 - mean_rewardDecrement*1) + (0.230 - mean_rewardDecrement*2) + ...
            (0.230 - mean_rewardDecrement*3) + (0.230 - mean_rewardDecrement*4))*(length(trialData)/5);
        patchMaster.E.optimal_totalReward_subjThreshold = (0.230 + (0.230 - mean_rewardDecrement*1) + ...
            (0.230 - mean_rewardDecrement*2) + (0.230 - mean_rewardDecrement*3) + ...
            (0.230 - mean_rewardDecrement*4) + (0.230 - mean_rewardDecrement*5) + ...
            (0.230 - mean_rewardDecrement*6) + (0.230 - mean_rewardDecrement*7) + ...
            (0.230 - mean_rewardDecrement*8) + ((0.230 - mean_rewardDecrement*9)*0.2627))*...
            (length(trialData)/patchMaster.E.patchLength_mean);
        patchMaster.E.actual_rewardRate = patchMaster.E.actual_totalReward / ...
            length(trialData);
        patchMaster.E.optimal_rewardRate = patchMaster.E.optimal_totalReward / length(trialData);
        patchMaster.E.optimal_rewardRate_subjThreshold = patchMaster.E.optimal_totalReward_subjThreshold / ...
            length(trialData);
        patchMaster.E.variabilityCost_perTrial = (patchMaster.E.optimal_totalReward_subjThreshold - ...
            patchMaster.E.actual_totalReward) / length(trialData);
        patchMaster.E.variabilityCost_perPatch = patchMaster.E.variabilityCost_perTrial * patchMaster.E.patchLength_mean;
        patchMaster.E.mean_rewardDecrement = mean_rewardDecrement;
        
        patchMaster.E.sessionData = charnovData;
        patchMaster.E.patchData = patches;
        patchMaster.E.trialData = trialData;
        patchMaster.E.ctrlFlag = 0;
        
        if youWannaSave
            cd(rootFolder)
            save('patchMaster','patchMaster');
        end
        
    end




end
