clear;
load('patchMaster.mat')

%% Options
if patchMaster.E.ctrlFlag == 1;
    cutoff_patches = 0;
else
    cutoff_patches = 1;
end
 
youWannaScatter = 0;
youWannaHist = 1; numBins = 8;
youWannaSave = 1;

%% Setup
patches_E = patchMaster.E.patchData;
trialTimes_E = [patchMaster.E.patchData.timePerTrial];
patchReward_E = [patches_E.reward]; patchLength_E = [patches_E.length];
mean_trialTime_E = mean(trialTimes_E); std_trialTime_E = std(trialTimes_E);
mean_patchLength_E = mean(patchLength_E); std_patchLength_E = std(patchLength_E);

%% Remove patches where the subject got distracted (super long patches)
% (just for the purposes of accurately estimating adjusted trial times)

if cutoff_patches
    
    % trial-time based cutoffs (for E and O only)
    cutoff_trialTime_E = mean_trialTime_E + (3 * std_trialTime_E);
    goodPatches_E = find([patchMaster.E.patchData.timePerTrial]'<cutoff_trialTime_E);
    patchMaster.E.patchData = patchMaster.E.patchData(goodPatches_E);
    
    % patch-length based cutoffs (for all 3 monkeys)
    cutoff_patchLength_E = mean_patchLength_E + (3 * std_patchLength_E);
    goodPatchLength_E = find([patchMaster.E.patchData.length]'<cutoff_patchLength_E);
    patchMaster.E.patchData = patchMaster.E.patchData(goodPatchLength_E);
    
    patchMaster.E.ctrlFlag = 1;
    save('patchMaster','patchMaster');
    
    % reload
    clearvars patchMaster
    load('patchMaster.mat')
    patches_E = patchMaster.E.patchData;
    trialTimes_E = [patchMaster.E.patchData.timePerTrial];
    patchReward_E = [patches_E.reward];
    patchLength_E = [patches_E.length];
    mean_trialTime_E = mean(trialTimes_E); std_trialTime_E = std(trialTimes_E);
    mean_patchLength_E = mean(patchLength_E); std_patchLength_E = std(patchLength_E);
    
end

patchReward_E = [patches_E.reward]; patchLength_E = [patches_E.length];

% patchMaster.E.mean_travelTime = nanmean([patchMaster.E.patchData.travelTime]);
% patchMaster.E.mean_adj_travelTime = nanmean([patchMaster.E.patchData.adj_travelTime]);
            

%% Plot summaries of behavior

if youWannaHist % Figure 2A
    figure; histogram(patchLength_E,numBins)
end

if youWannaScatter
    figure(2); scatter(patchLength_E,patchReward_E);
end
%% Calculate adjusted travel times

adj_travelTimes = struct;
patchLengths_E = [patchMaster.E.patchData.length]';
adj_travelTime_E = [patchMaster.E.patchData.adj_travelTime]';
adj_patchTravel(:,1) = [patchLengths_E];
adj_patchTravel(:,2) = [adj_travelTime_E];
for Smax = 1:18
    ix_S = find(adj_patchTravel(:,1) == Smax);
    adj_travelTimes_byPatchLength(Smax,1:length(ix_S)) = adj_patchTravel(ix_S,2);
    adj_travelTimes(Smax).patchLength = Smax;
    adj_travelTimes(Smax).mean_adj_travelTime = nanmean(adj_travelTimes_byPatchLength(Smax,:));
    adj_travelTimes(Smax).STD_adj_travelTime = nanstd(adj_travelTimes_byPatchLength(Smax,:));
end
patchMaster.Global.adj_travelTimes = adj_travelTimes;
patchMaster.Global.adj_travelTimes_raw = adj_travelTimes_byPatchLength;

adj_mean_travelTimes = ([patchMaster.Global.adj_travelTimes.mean_adj_travelTime]');
adj_STD_travelTimes = ([patchMaster.Global.adj_travelTimes.STD_adj_travelTime]');

smooth_adj_mean_travelTimes = smooth(adj_mean_travelTimes);
smooth_adj_STD_travelTimes = smooth(adj_STD_travelTimes);

for Smax = 1:18
    patchMaster.Global.adj_travelTimes(Smax).smooth_mean_adj = smooth_adj_mean_travelTimes(Smax);
    patchMaster.Global.adj_travelTimes(Smax).smooth_STD_adj = smooth_adj_STD_travelTimes(Smax);
end

%% Calculate overall # trials per session (for use in simulations)
% sessionTrials(1) = patchMaster.C.total_numTrials;
% sessions(1) = size(patchMaster.C.sessionData,2); 
% sessionTrials(2) = patchMaster.E.total_numTrials;
% sessions(2) = size(patchMaster.E.sessionData,2);
% sessionTrials(3) = patchMaster.O.total_numTrials;
% sessions(3) = size(patchMaster.O.sessionData,2);
% for i = 1:3
%    trials_perSession(i) = sessionTrials(i)/sessions(i);
%    weight(i) = sessions(i)/sum(sessions);
%    weighted_trialsPerSession = trials_perSession(i) * weight(i);
% end
patchMaster.Global.trials_perSession = 437; %empirical %round(sum(sessionTrials)/sum(sessions));

%% Save
if youWannaSave, save('patchMaster','patchMaster'); end
clear;